@@ -1,11 +1,12 @@
-{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE CPP               #-}
+{-# LANGUAGE DeriveAnyClass    #-}
+{-# LANGUAGE DeriveGeneric     #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE DeriveGeneric, DeriveAnyClass #-}
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE Rank2Types        #-}
 
 -- |
 -- Module    : Data.Versions
--- Copyright : (c) Colin Woodbury, 2015 - 2018
+-- Copyright : (c) Colin Woodbury, 2015 - 2019
 -- License   : BSD3
 -- Maintainer: Colin Woodbury <colingw@gmail.com>
 --
@@ -13,10 +14,9 @@
 --
 -- We like to give version numbers to our software in a myriad of different
 -- ways. Some ways follow strict guidelines for incrementing and comparison.
--- Some follow conventional wisdom and are generally self-consistent.
--- Some are just plain asinine. This library provides a means of parsing
--- and comparing /any/ style of versioning, be it a nice Semantic Version
--- like this:
+-- Some follow conventional wisdom and are generally self-consistent. Some are
+-- just plain asinine. This library provides a means of parsing and comparing
+-- /any/ style of versioning, be it a nice Semantic Version like this:
 --
 -- > 1.2.3-r1+git123
 --
@@ -24,16 +24,15 @@
 --
 -- > 2:10.2+0.0093r3+1-1
 --
--- Please switch to <http://semver.org Semantic Versioning> if you
--- aren't currently using it. It provides consistency in version
--- incrementing and has the best constraints on comparisons.
+-- Please switch to <http://semver.org Semantic Versioning> if you aren't
+-- currently using it. It provides consistency in version incrementing and has
+-- the best constraints on comparisons.
 --
 -- == Using the Parsers
--- In general, `versioning` is the function you want. It attempts to parse
--- a given @Text@ using the three individual parsers, `semver`, `version`
--- and `mess`. If one fails, it tries the next. If you know you only want
--- to parse one specific version type, use that parser directly
--- (e.g. `semver`).
+-- In general, `versioning` is the function you want. It attempts to parse a
+-- given @Text@ using the three individual parsers, `semver`, `version` and
+-- `mess`. If one fails, it tries the next. If you know you only want to parse
+-- one specific version type, use that parser directly (e.g. `semver`).
 
 module Data.Versions
     (
@@ -52,7 +51,7 @@ module Data.Versions
       -- | For when you'd like to mix version parsing into some larger parser.
     , versioning', semver', version', mess'
       -- * Pretty Printing
-    , prettyV, prettySemVer, prettyVer, prettyMess, parseErrorPretty
+    , prettyV, prettySemVer, prettyVer, prettyMess, errorBundlePretty
       -- * Lenses
     , Lens'
     , Traversal'
@@ -78,31 +77,32 @@ import qualified Data.Text as T
 import           Data.Void
 import           Data.Word (Word)
 import           GHC.Generics
-import           Text.Megaparsec
+import           Text.Megaparsec hiding (chunk)
 import           Text.Megaparsec.Char
+import qualified Text.Megaparsec.Char.Lexer as L
 
-#if __GLASGOW_HASKELL__ < 841
-import Data.Semigroup
+#if !MIN_VERSION_base(4,11,0)
+import           Data.Semigroup
 #endif
 
 ---
 
--- | A top-level Versioning type. Acts as a wrapper for the more specific
--- types. This allows each subtype to have its own parser, and for said
--- parsers to be composed. This is useful for specifying custom behaviour
--- for when a certain parser fails.
+-- | A top-level Versioning type. Acts as a wrapper for the more specific types.
+-- This allows each subtype to have its own parser, and for said parsers to be
+-- composed. This is useful for specifying custom behaviour for when a certain
+-- parser fails.
 data Versioning = Ideal SemVer | General Version | Complex Mess
   deriving (Eq,Show,Generic,NFData,Hashable)
 
 -- | Comparison of @Ideal@s is always well defined.
 --
--- If comparison of @General@s is well-defined, then comparison
--- of @Ideal@ and @General@ is well-defined, as there exists a perfect
--- mapping from @Ideal@ to @General@.
+-- If comparison of @General@s is well-defined, then comparison of @Ideal@ and
+-- @General@ is well-defined, as there exists a perfect mapping from @Ideal@ to
+-- @General@.
 --
 -- If comparison of @Complex@es is well-defined, then comparison of @General@
--- and @Complex@ is well defined for the same reason.
--- This implies comparison of @Ideal@ and @Complex@ is also well-defined.
+-- and @Complex@ is well defined for the same reason. This implies comparison of
+-- @Ideal@ and @Complex@ is also well-defined.
 instance Ord Versioning where
   compare (Ideal s)     (Ideal s')    = compare s s'
   compare (General v)   (General v')  = compare v v'
@@ -181,17 +181,17 @@ _Mess f t = either (const (pure t)) (fmap prettyMess . f) $ mess t
 
 _Ideal :: Traversal' Versioning SemVer
 _Ideal f (Ideal s) = Ideal <$> f s
-_Ideal _ v = pure v
+_Ideal _ v         = pure v
 {-# INLINE _Ideal #-}
 
 _General :: Traversal' Versioning Version
 _General f (General v) = General <$> f v
-_General _ v = pure v
+_General _ v           = pure v
 {-# INLINE _General #-}
 
 _Complex :: Traversal' Versioning Mess
 _Complex f (Complex m) = Complex <$> f m
-_Complex _ v = pure v
+_Complex _ v           = pure v
 {-# INLINE _Complex #-}
 
 -- | Simple Lenses compatible with both lens and microlens.
@@ -200,13 +200,13 @@ type Lens' s a = forall f. Functor f => (a -> f a) -> s -> f s
 -- | Simple Traversals compatible with both lens and microlens.
 type Traversal' s a = forall f. Applicative f => (a -> f a) -> s -> f s
 
--- | Version types which sanely and safely yield `SemVer`-like information
--- about themselves. For instances other than `SemVer` itself however,
--- these optics may /not/ yield anything, depending on the actual value being
--- traversed. Hence, the optics here are all `Traversal'`s.
+-- | Version types which sanely and safely yield `SemVer`-like information about
+-- themselves. For instances other than `SemVer` itself however, these optics
+-- may /not/ yield anything, depending on the actual value being traversed.
+-- Hence, the optics here are all `Traversal'`s.
 --
--- Consider the `Version` @1.2.3.4.5@. We can imagine wanting to increment
--- the minor number:
+-- Consider the `Version` @1.2.3.4.5@. We can imagine wanting to increment the
+-- minor number:
 --
 -- @
 -- λ "1.2.3.4.5" & minor %~ (+ 1)
@@ -317,9 +317,9 @@ instance Semantic SemVer where
   semantic = ($)
   {-# INLINE semantic #-}
 
--- | A single unit of a Version. May be digits or a string of characters.
--- Groups of these are called `VChunk`s, and are the identifiers separated
--- by periods in the source.
+-- | A single unit of a Version. May be digits or a string of characters. Groups
+-- of these are called `VChunk`s, and are the identifiers separated by periods
+-- in the source.
 data VUnit = Digits Word | Str T.Text deriving (Eq,Show,Read,Ord,Generic,NFData,Hashable)
 
 instance Semigroup VUnit where
@@ -345,12 +345,12 @@ str t = bool Nothing (Just $ Str t) $ T.all isAlpha t
 
 _Digits :: Traversal' VUnit Word
 _Digits f (Digits i) = Digits <$> f i
-_Digits _ v = pure v
+_Digits _ v          = pure v
 {-# INLINE _Digits #-}
 
 _Str :: Traversal' VUnit T.Text
 _Str f (Str t) = Str . (\t' -> bool t t' (T.all isAlpha t')) <$> f t
-_Str _ v = pure v
+_Str _ v       = pure v
 {-# INLINE _Str #-}
 
 -- | A logical unit of a version number. Can consist of multiple letters
@@ -408,21 +408,22 @@ instance Ord Version where
 
   -- | If one side has run out of chunks to compare but the other hasn't,
   -- the other must be newer.
-  compare (Version _ _ _)  (Version _ [] _) = GT
-  compare (Version _ [] _) (Version _ _ _)  = LT
+  compare Version{}  (Version _ [] _) = GT
+  compare (Version _ [] _) Version{}  = LT
 
-  -- | The usual case. If first VChunks of each Version is equal, then we
-  -- keep recursing. Otherwise, we don't need to check further. Consider @1.2@
+  -- | The usual case. If first VChunks of each Version is equal, then we keep
+  -- recursing. Otherwise, we don't need to check further. Consider @1.2@
   -- compared to @1.1.3.4.5.6@.
   compare (Version _ (a:as) rs) (Version _ (b:bs) rs') = case f a b of
     EQ  -> compare (Version Nothing as rs) (Version Nothing bs rs')
     res -> res
     where f [] [] = EQ
 
-          -- | Opposite of the above. If we've recursed this far and one side has
-          -- fewer chunks, it must be the "greater" version. A Chunk break only occurs in
-          -- a switch from digits to letters and vice versa, so anything "extra" must be
-          -- an @rc@ marking or similar. Consider @1.1@ compared to @1.1rc1@.
+          -- | Opposite of the above. If we've recursed this far and one side
+          -- has fewer chunks, it must be the "greater" version. A Chunk break
+          -- only occurs in a switch from digits to letters and vice versa, so
+          -- anything "extra" must be an @rc@ marking or similar. Consider @1.1@
+          -- compared to @1.1rc1@.
           f [] _  = GT
           f _ []  = LT
 
@@ -467,18 +468,17 @@ epoch :: Lens' Version (Maybe Word)
 epoch f v = fmap (\ve -> v { _vEpoch = ve }) (f $ _vEpoch v)
 {-# INLINE epoch #-}
 
--- | A (Complex) Mess.
--- This is a /descriptive/ parser, based on examples of stupidly
--- crafted version numbers used in the wild.
+-- | A (Complex) Mess. This is a /descriptive/ parser, based on examples of
+-- stupidly crafted version numbers used in the wild.
 --
--- Groups of letters/numbers, separated by a period, can be
--- further separated by the symbols @_-+:@
+-- Groups of letters/numbers, separated by a period, can be further separated by
+-- the symbols @_-+:@
 --
--- Unfortunately, @VChunk@s cannot be used here, as some developers have
--- numbers like @1.003.04@ which make parsers quite sad.
+-- Unfortunately, @VChunk@s cannot be used here, as some developers have numbers
+-- like @1.003.04@ which make parsers quite sad.
 --
--- Not guaranteed to have well-defined ordering (@Ord@) behaviour,
--- but so far internal tests show consistency.
+-- Not guaranteed to have well-defined ordering (@Ord@) behaviour, but so far
+-- internal tests show consistency.
 data Mess = VLeaf [T.Text] | VNode [T.Text] VSep Mess deriving (Eq,Show,Generic,NFData,Hashable)
 
 instance Ord Mess where
@@ -522,8 +522,8 @@ instance Semantic Mess where
   semantic _ v = pure v
   {-# INLINE semantic #-}
 
--- | Developers use a number of symbols to seperate groups of digits/letters
--- in their version numbers. These are:
+-- | Developers use a number of symbols to seperate groups of digits/letters in
+-- their version numbers. These are:
 --
 -- * A colon (:). Often denotes an "epoch".
 -- * A hyphen (-).
@@ -532,16 +532,19 @@ instance Semantic Mess where
 data VSep = VColon | VHyphen | VPlus | VUnder deriving (Eq,Show,Generic,NFData,Hashable)
 
 -- | A synonym for the more verbose `megaparsec` error type.
-type ParsingError = ParseError (Token T.Text) Void
+type ParsingError = ParseErrorBundle T.Text Void
 
 -- | Parse a piece of `T.Text` into either an (Ideal) `SemVer`, a (General)
 -- `Version`, or a (Complex) `Mess`.
 versioning :: T.Text -> Either ParsingError Versioning
 versioning = parse versioning' "versioning"
 
--- | Parse a `Versioning`. Assumes that the version number is the last token in the string.
+-- | Parse a `Versioning`. Assumes the version number is the last token in
+-- the string.
 versioning' :: Parsec Void T.Text Versioning
-versioning' = try (fmap Ideal semver' <* eof) <|> try (fmap General version' <* eof) <|> (fmap Complex mess' <* eof)
+versioning' = choice [ try (fmap Ideal semver'    <* eof)
+                     , try (fmap General version' <* eof)
+                     , fmap Complex mess'         <* eof ]
 
 -- | Parse a (Ideal) Semantic Version.
 semver :: T.Text -> Either ParsingError SemVer
@@ -549,7 +552,7 @@ semver = parse (semver' <* eof) "Semantic Version"
 
 -- | Internal megaparsec parser of `semver`.
 semver' :: Parsec Void T.Text SemVer
-semver' = SemVer <$> majorP <*> minorP <*> patchP <*> preRel <*> metaData
+semver' = L.lexeme space (SemVer <$> majorP <*> minorP <*> patchP <*> preRel <*> metaData)
 
 -- | Parse a group of digits, which can't be lead by a 0, unless it is 0.
 digitsP :: Parsec Void T.Text Word
@@ -596,7 +599,7 @@ version = parse (version' <* eof) "Version"
 
 -- | Internal megaparsec parser of `version`.
 version' :: Parsec Void T.Text Version
-version' = Version <$> optional (try epochP) <*> chunks <*> preRel
+version' = L.lexeme space (Version <$> optional (try epochP) <*> chunks <*> preRel)
 
 epochP :: Parsec Void T.Text Word
 epochP = read <$> (some digitChar <* char ':')
@@ -607,7 +610,7 @@ mess = parse (mess' <* eof) "Mess"
 
 -- | Internal megaparsec parser of `mess`.
 mess' :: Parsec Void T.Text Mess
-mess' = try node <|> leaf
+mess' = L.lexeme space (try node <|> leaf)
 
 leaf :: Parsec Void T.Text Mess
 leaf = VLeaf <$> tchunks
