@@ -1,4 +1,7 @@
-{-# LANGUAGE OverloadedStrings, NoImplicitPrelude #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE OverloadedStrings #-}
+
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Main where
 
@@ -27,8 +30,8 @@ chunks = resize 10 . listOf1 . fmap simplify . resize 10 $ listOf1 arbitrary
 simplify :: [VUnit] -> [VUnit]
 simplify = map fold . groupBy f
   where f (Digits _) (Digits _) = True
-        f (Str _) (Str _) = True
-        f _ _ = False
+        f (Str _) (Str _)       = True
+        f _ _                   = False
 
 instance EqProp SemVer where
   a =-= b = eq a b
@@ -98,13 +101,13 @@ suite :: TestTree
 suite = testGroup "Tests"
   [ testGroup "Property Tests"
     [ testGroup "SemVer - Monoid" $
-      map (\(name, test) -> testProperty name test) . unbatch $ monoid (SemVer 1 2 3 [] [])
+      map (uncurry testProperty) . unbatch $ monoid (SemVer 1 2 3 [] [])
     , testProperty "SemVer - Arbitrary" $ \a -> isRight . fmap (== a) $ semver (prettySemVer a)
     , testProperty "Version - Arbitrary" $ \a -> isRight . fmap (== a) $ version (prettyVer a)
     -- , testGroup "Version - Monoid" $
     --   map (\(name, test) -> testProperty name test) . unbatch $ monoid (Version (Just 1) [[digits 2], [digits 3]])
     , testGroup "VUnit - Monoid" $
-      map (\(name, test) -> testProperty name test) . unbatch $ monoid (Digits 0)
+      map (uncurry testProperty) . unbatch $ monoid (Digits 0)
     ]
   , testGroup "Unit Tests"
     [ testGroup "(Ideal) Semantic Versioning"
@@ -118,6 +121,9 @@ suite = testGroup "Tests"
          $ semver "1.2.3-alpha.2" == semver "1.2.3-alpha.2+a1b2c3.1") :
         map (\(a,b) -> testCase (T.unpack $ a <> " < " <> b) $ comp semver a b)
         (zip semverOrd $ tail semverOrd)
+      , testGroup "Whitespace Handling"
+        [ testCase "1.2.3-1[ ]" $ parse semver' "semver whitespace" "1.2.3-1 " @?= Right (SemVer 1 2 3 [[Digits 1]] [])
+        ]
       ]
     , testGroup "(General) Versions"
       [ testGroup "Good Versions" $
@@ -196,15 +202,15 @@ check = assertBool "Some Either-based assertion failed" . either (const False) i
 
 isSemVer :: Versioning -> Bool
 isSemVer (Ideal _) = True
-isSemVer _ = False
+isSemVer _         = False
 
 isVersion :: Versioning -> Bool
 isVersion (General _) = True
-isVersion _ = False
+isVersion _           = False
 
 isMess :: Versioning -> Bool
 isMess (Complex _) = True
-isMess _ = False
+isMess _           = False
 
 incPatch :: Assertion
 incPatch = (v1 & patch %~ (+ 1)) @?= v2
@@ -227,10 +233,10 @@ main :: IO ()
 main = defaultMain suite
 
 nameGrab :: Parsec Void T.Text T.Text
-nameGrab = T.pack <$> manyTill anyChar (try finished)
+nameGrab = T.pack <$> manyTill anySingle (try finished)
   where finished = char '-' *> lookAhead digitChar
 
 versionGrab :: Parsec Void T.Text Versioning
-versionGrab = manyTill anyChar (try finished) *> ver
+versionGrab = manyTill anySingle (try finished) *> ver
   where finished = char '-' *> lookAhead digitChar
         ver = fmap Ideal semver' <|> fmap General version' <|> fmap Complex mess'
